# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class QuoteInstrumentIdType(str, Enum):
    """
    QuoteInstrumentIdType
    """

    """
    allowed enum values
    """
    LUSIDINSTRUMENTID = 'LusidInstrumentId'
    FIGI = 'Figi'
    RIC = 'RIC'
    QUOTEPERMID = 'QuotePermId'
    ISIN = 'Isin'
    CURRENCYPAIR = 'CurrencyPair'
    CLIENTINTERNAL = 'ClientInternal'
    SEDOL = 'Sedol'
    CUSIP = 'Cusip'

    @classmethod
    def from_json(cls, json_str: str) -> QuoteInstrumentIdType:
        """Create an instance of QuoteInstrumentIdType from a JSON string"""
        return QuoteInstrumentIdType(json.loads(json_str))
