# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class AccountingMethod(str, Enum):
    """
    AccountingMethod
    """

    """
    allowed enum values
    """
    DEFAULT = 'Default'
    AVERAGECOST = 'AverageCost'
    FIRSTINFIRSTOUT = 'FirstInFirstOut'
    LASTINFIRSTOUT = 'LastInFirstOut'
    HIGHESTCOSTFIRST = 'HighestCostFirst'
    LOWESTCOSTFIRST = 'LowestCostFirst'

    @classmethod
    def from_json(cls, json_str: str) -> AccountingMethod:
        """Create an instance of AccountingMethod from a JSON string"""
        return AccountingMethod(json.loads(json_str))
