# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class MarketDataOptionsType(str, Enum):
    """
    The type of MarketDataOptions provided
    """

    """
    allowed enum values
    """
    CURVEOPTIONS = 'CurveOptions'

    @classmethod
    def from_json(cls, json_str: str) -> MarketDataOptionsType:
        """Create an instance of MarketDataOptionsType from a JSON string"""
        return MarketDataOptionsType(json.loads(json_str))
