# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class ReconciliationRuleType(str, Enum):
    """
    Types of available reconciliation rule.
    """

    """
    allowed enum values
    """
    RECONCILENUMERICRULE = 'ReconcileNumericRule'
    RECONCILEDATETIMERULE = 'ReconcileDateTimeRule'
    RECONCILESTRINGRULE = 'ReconcileStringRule'
    RECONCILEEXACT = 'ReconcileExact'

    @classmethod
    def from_json(cls, json_str: str) -> ReconciliationRuleType:
        """Create an instance of ReconciliationRuleType from a JSON string"""
        return ReconciliationRuleType(json.loads(json_str))
