# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict
from pydantic.v1 import Field, StrictStr, constr, validator
from lusid.models.match_criterion import MatchCriterion

class SubHoldingKeyValueEquals(MatchCriterion):
    """
    A criterion that checks whether a SubHoldingKey Value is equal to the given string value  # noqa: E501
    """
    sub_holding_key: StrictStr = Field(..., alias="subHoldingKey", description="The sub holding key whose value will form the left-hand side of the operation")
    value: constr(strict=True, max_length=1024, min_length=0) = Field(..., description="The value to be compared against")
    criterion_type: StrictStr = Field(..., alias="criterionType", description="The available values are: PropertyValueEquals, PropertyValueIn, SubHoldingKeyValueEquals")
    additional_properties: Dict[str, Any] = {}
    __properties = ["criterionType", "subHoldingKey", "value"]

    @validator('criterion_type')
    def criterion_type_validate_enum(cls, value):
        """Validates the enum"""
        if value not in ('PropertyValueEquals', 'PropertyValueIn', 'SubHoldingKeyValueEquals'):
            raise ValueError("must be one of enum values ('PropertyValueEquals', 'PropertyValueIn', 'SubHoldingKeyValueEquals')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> SubHoldingKeyValueEquals:
        """Create an instance of SubHoldingKeyValueEquals from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "additional_properties"
                          },
                          exclude_none=True)
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> SubHoldingKeyValueEquals:
        """Create an instance of SubHoldingKeyValueEquals from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return SubHoldingKeyValueEquals.parse_obj(obj)

        _obj = SubHoldingKeyValueEquals.parse_obj({
            "criterion_type": obj.get("criterionType"),
            "sub_holding_key": obj.get("subHoldingKey"),
            "value": obj.get("value")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj
