# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class MarketObservableType(str, Enum):
    """
    MarketObservableType
    """

    """
    allowed enum values
    """
    FORWARDPOINTS = 'ForwardPoints'
    FORWARDRATE = 'ForwardRate'
    RATESCURVE = 'RatesCurve'
    FXFORWARDCURVE = 'FxForwardCurve'
    INVALID = 'Invalid'

    @classmethod
    def from_json(cls, json_str: str) -> MarketObservableType:
        """Create an instance of MarketObservableType from a JSON string"""
        return MarketObservableType(json.loads(json_str))
