# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class AssetClass(str, Enum):
    """
    AssetClass
    """

    """
    allowed enum values
    """
    INTERESTRATES = 'InterestRates'
    FX = 'FX'
    INFLATION = 'Inflation'
    EQUITIES = 'Equities'
    CREDIT = 'Credit'
    COMMODITIES = 'Commodities'
    MONEY = 'Money'
    UNKNOWN = 'Unknown'

    @classmethod
    def from_json(cls, json_str: str) -> AssetClass:
        """Create an instance of AssetClass from a JSON string"""
        return AssetClass(json.loads(json_str))
