# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class AggregationOp(str, Enum):
    """
    AggregationOp
    """

    """
    allowed enum values
    """
    SUM = 'Sum'
    PROPORTION = 'Proportion'
    AVERAGE = 'Average'
    COUNT = 'Count'
    MIN = 'Min'
    MAX = 'Max'
    VALUE = 'Value'
    SUMOFPOSITIVEVALUES = 'SumOfPositiveValues'
    SUMOFNEGATIVEVALUES = 'SumOfNegativeValues'
    SUMOFABSOLUTEVALUES = 'SumOfAbsoluteValues'
    PROPORTIONOFABSOLUTEVALUES = 'ProportionOfAbsoluteValues'
    SUMCUMULATIVEINADVANCE = 'SumCumulativeInAdvance'
    SUMCUMULATIVEINARREARS = 'SumCumulativeInArrears'

    @classmethod
    def from_json(cls, json_str: str) -> AggregationOp:
        """Create an instance of AggregationOp from a JSON string"""
        return AggregationOp(json.loads(json_str))
